;+
; NAME:
; 
;	COM_FIND
;
; PURPOSE:
; 
;       Return the 'center-of-mass' of the supplied data array.
;
; CALLING SEQUENCE:
; 
;	Result = COM_FIND(X,Y)
;
; INPUTS:
; 
;       X, Y - 1D data arrays.
;
; OUTPUTS:
; 
;       Result = The X value corresponding to the center of mass,
;       i.e., the X value that divides the integral of Y in half.
;       Specifically, if X_com is the center-of-mass value, then
;       
;       Integral(Y)_from_0_to_X_com = Integral(Y)_from_X_com_to_MAX(X)
;
; KEYWORD PARAMETERS:
; 
;       INTERPOLATE - Set this keyword to return a 'floating-point'
;       index instead of an integer value.  
;
;       X_com=COM_FIND(X,Y,/INTERPOLATE)
;
; EXAMPLE:
;
;       Make some noisy data:
;
;       x=VECTOR(0.,5.,100)
;       y=SIN(x)+0.1*RANDOMN(seed,100)
;
;       Determine the center of mass:
;
;	x_com=COM_FIND(y)
;
; MODIFICATION HISTORY:
; 
; 	David L. Windt, December 2003
; 	
;       windt@astro.columbia.edu
;                    
;-
function com_find,x,y,interpolate=interpolate

;; return if the data is flat or zero:
if (total(y) eq 0) or (total(where(y ne y[0])) eq -1) $
  then return,x[0]

;; compute an array of totals:
int=total(y-min(y),/cumulative)

if (keyword_set(interpolate) eq 0) or $
  (total(where(finite(int) ne 1)) ne -1) then begin
    ;; don't interpolate:
    x_com=x[max(where(int le max(int)*.5))>0]
endif else begin
    ;; interpolate to get the corresponding X value:
    x_com=interpolate(x,findex(int,max(int)*.5))
endelse

return,x_com
end
