;+
; NAME:
; 
;       CW_LEGEND_RXO
;
; PURPOSE:
; 
;       A compound widget used to set values for the POSITION, NOBOX,
;       and BOXFILL keywords to the LEGEND_RXO procedure.
;       
;      
; CATEGORY:
; 
;	Compound widgets.
;
; CALLING SEQUENCE:
; 
;	Result = CW_LEGEND_RXO(PARENT,LABEL)
;
; INPUTS:
; 
;       PARENT - The ID of the parent widget.  
;
;       LABEL - a label to be drawn to the left (or top, for /column)
;               of the widget.
;
; OPTIONAL KEYWORD PARAMETERS:
;
;       ADD_LABEL_CURVE_OPTION - set to add "Label Curves" as an
;                                additional, last option in the list
;                                of legend positions. (Return value
;                                for Label Curves is 13.)
; 
;	UVALUE - Supplies the user value for the widget.
;
;       VALUE - an array of initial values:
;               [POSITION, NOBOX, BOXFILL]
;
;       FRAME - set to draw a frame around the widget.
;       
;       FONT - font keyword for labels etc.
;
;       XPAD, YPAD, SPACE - keywords to widget_base
;
;
; OUTPUTS:
; 
;       The ID of the created widget is returned.
;
; COMMON BLOCKS:
;
;       CW_PLOTSTYLE: private common block containing color bitmaps
;       for 'buttons' and menus, and arrays of valid values for thick,
;       psym and symsize.
;
; RESTRICTIONS:
;
;       Uses the cgImage command in place of the tv command to display
;       images correctly on all devices. cgImage is part of the Coyote
;       Graphics System at www.idlcoyote.com.
;
; PROCEDURE/EXAMPLE:
;
; MODIFICATION HISTORY:
; 
;       David L. Windt, Reflective X-ray Optics, May 2013
;       davidwindt@gmail.com.
;
; -

;------------------------------------------------------------------
; event handler
function cw_legend_rxo_ev,event

compile_opt hidden

on_error,2

common cw_legend_rxo,position_bm,boxfill_bm

;; retrieve the state, which contains the widget id's
widget_control,widget_info(event.handler,/child),get_uvalue=s

if tag_names(event,/str) eq 'WIDGET_DRAW' then begin

    ;; menu base:
    menu_base=widget_info(event.id,/child)

    ;; create context menu if it doesn't exist:
    if widget_info(menu_base,/child) eq 0L then begin

        ;; which menu:
        case event.id of
            s.position: begin
                bm=position_bm[*,*,*,0:12+s.add_label_curve_option]
                uvalue_base='position_'
            end
            s.boxfill: begin
                bm=boxfill_bm
                uvalue_base='boxfill_'
            end
        endcase

        ;; how many:
        n=(size(bm))[4]

        ;; make a button for each:
        ;; if event.id=s.position, ands.no_below, then omit positions 1,2 and 3:

        for j=0,n-1 do begin
            make_button=1
            if (event.id eq s.position) and (s.no_below) then $
              if ((j eq 1) or (j eq 2) or (j eq 3)) then make_button=0
            if make_button then $
              button=widget_button(menu_base, $
                                   value=bm[*,*,*,j], $
                                   uvalue=uvalue_base+strtrim(j,2))
        endfor
    endif

    ;; display context menu:
    widget_displaycontextmenu, $
      event.id,event.x,event.y,menu_base

    eventval=0
    goto,done

endif

;; get eventval:
widget_control,event.id,get_uvalue=uvalue

;; get value:
if n_elements(uvalue) eq 1 then begin

    case 1 of
        strmid(uvalue,0,9) eq 'position_': begin
            ;; get position index:
            position=(fix(strmid(uvalue,9)) < ((size(position_bm))[4]-1)) > 0
            widget_control,s.position, $
              get_value=wid, $
              set_uvalue=position

            eventval=position
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,position_bm[*,*,*,position]
            endif
        end
        strmid(uvalue,0,8) eq 'boxfill_': begin
            ;; get boxfill index:
            boxfill=((fix(strmid(uvalue,8)) < ((size(boxfill_bm))[4]-1)) > 0)-1
            widget_control,s.boxfill, $
              get_value=wid, $
              set_uvalue=boxfill

            eventval=boxfill
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,boxfill_bm[*,*,*,boxfill+1]
            endif
        end
        uvalue eq 'nobox': begin
            ;; get value:
            nobox=event.select eq 0
            eventval=nobox
        end
        else: eventval=0
    endcase
endif

done: ;;

return,{CW_PLOTSTYLE_EVENT,ID:event.handler,TOP:event.top,HANDLER:0L,TAG:eventval}
end

;------------------------------------------------------------------
; set value procedure
pro cw_legend_rxo_set_value,id,value

compile_opt hidden

on_error,2

common cw_legend_rxo

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; make sure values are in range:
position=(value[0] < ((size(position_bm))[4]-1)) > 0
nobox=(value[1] < 1) > 0
boxfill=(value[2] < ((size(boxfill_bm))[4]-2)) > (-1)

;; set the widget values:

;; position:
widget_control,s.position,set_uvalue=position,get_value=wid
if (!d.flags and 256) eq 256 then $
  if wid gt 0 then begin
    wset,wid
    cgimage,position_bm[*,*,*,position]
endif

;; nobox:
widget_control,s.nobox,set_value=nobox eq 0

;; boxfill:
widget_control,s.boxfill,set_uvalue=boxfill, $
  sensitive=(nobox eq 0),get_value=wid
if (!d.flags and 256) eq 256 then $
  if wid gt 0 then begin
    wset,wid
    cgimage,boxfill_bm[*,*,*,boxfill+1]
endif

return
end

;------------------------------------------------------------------
; get value function
function cw_legend_rxo_get_value,id

compile_opt hidden
on_error,2

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; get the widget values:

;; position:
widget_control,s.position,get_uvalue=position

;; nobox:
widget_control,s.nobox,get_value=val
nobox=val eq 0

;; boxfill:
widget_control,s.boxfill,get_uvalue=boxfill

return,[position,nobox,boxfill]
end

;------------------------------------------------------------------
; main function
function cw_legend_rxo,parent,label, $
           add_label_curve_option=add_label_curve_option, $
           no_below=no_below, $
           value=value, $
           uvalue=uvalue, $
           frame=frame, $
           font=font, $
           row=row,column=column, $
           xpad=xpad,ypad=ypad,space=space

on_error,2

common cw_legend_rxo

;; bitmap sizes:
xsize=40
ysize=10
xpsize=28
ypsize=22

;; default values:
n_boxfill=33
n_position=14

;; create bitmaps if undefined:
sz=size(boxfill_bm)
init=(sz[0] eq 0) ? 1 : (sz[4] ne n_boxfill)
if init then begin
    ;; get current color table:
    tvlct,r,g,b,/get

    ;; colors (loaded from current color table):
    boxfill_bm=bytarr(xsize,ysize,3,n_boxfill)
    for j=0,n_boxfill-2 do begin
        boxfill_bm[*,*,0,j+1]=r[j]
        boxfill_bm[*,*,1,j+1]=g[j]
        boxfill_bm[*,*,2,j+1]=b[j]
    endfor

    ;; make pixmap for creating "no fill" bitmap:
    window,/pixmap,/free,xsize=xsize,ysize=ysize

    ;; save a copy of the current color table:
    r_save=r
    g_save=g
    b_save=b

    ;; create black and white in first to color indices:
    r[0]=0
    g[0]=0
    b[0]=0
    r[1]=255
    g[1]=255
    b[1]=255

    ;; load new color table
    tvlct,r,g,b

    ;; create 'no fill' boxfill_bm:

    ;; erase:
    erase,color=1
    
    xyouts,.5,.05,/norm,alignment=0.5,'No Fill',charsize=1,color=0
    
    ;; save image:
    boxfill_bm[*,*,*,0]=tvrd(true=3)

    ;; positions:
    position_bm=bytarr(xpsize,ypsize,3,n_position)

    ;; plot coords to draw legend boxes:

    ;; plots coords to draw plot box:
    xp=[.05,.95,.95,.05,.05]
    yp=[.25,.25,.95,.95,.25]

    ;; plots coords to draw legend box:
    w=.25
    h=.2
    xx=[0,1,1,0,0]*w
    yy=[0,0,1,1,0]*h

    xo=[.05,.38,.7, $
        .05,.38,.7, $
        .05,.38,.7, $
        .05,.38,.7]

    yo=[.01,.01,.01, $
        .3,.3,.3, $
        .5,.5,.5, $
        .7,.7,.7]

    ;; delete pixmap:
    wdelete

    ;; make pixmap for creating position bitmaps:
    window,/pixmap,/free,xsize=xpsize,ysize=ypsize

    for j=0,n_position-1 do begin

        ;; erase:
        erase,color=1

        case j of

            ;; no legend:
            0: xyouts,.5,.5,/norm,alignment=0.5,'None',charsize=1,color=0

            ;; curve-label:
            13: xyouts,.5,.5,/norm,alignment=0.5,'Label!cCurves',charsize=.8,color=0

            ;; legend:
            else: begin

                ;; draw plot:
                plots,xp,yp,/norm,color=0

                ;; draw legend box:
                polyfill, $
                  xo[j-1]+xx, $
                  yo[j-1]+yy, $
                  /norm,color=0
            endelse
        endcase

        ;; save image:
        position_bm[*,*,*,j]=tvrd(true=3)
    endfor

    ;; delete pixmap:
    wdelete

    ;; restore old color table
    tvlct,r_save,g_save,b_save
endif

if n_params() eq 0 then message,'Must specify a parent for cw_legend_rxo.'
if not keyword_set(uvalue) then uvalue=0
if keyword_set(column) eq 0 then row=1

;; check that value is set properly
if n_elements(value) ne 5 then value=[0,0,-1]

value=[(value[0] < (n_position-1)) > 0, $
       (value[1] < 1) > 0, $
       (value[2] < (n_boxfill-1)) > (-1)]

; keep widget id's in state:
s={position:0L, $
   position_menu:0L, $
   add_label_curve_option:keyword_set(add_label_curve_option), $
   no_below:keyword_set(no_below), $
   nobox:0L, $
   boxfill:0L, $
   boxfill_menu:0L}

if n_elements(font) ne 1 then font=''

;; base:
base=widget_base(parent, $
                 uvalue=uvalue, $
                 frame=keyword_set(frame), $
                 event_func='cw_legend_rxo_ev', $
                 pro_set_value='cw_legend_rxo_set_value', $
                 func_get_value='cw_legend_rxo_get_value', $
                 xpad=xpad,ypad=ypad,space=space) 

base1=widget_base(base, $
                  row=keyword_set(row), $
                  column=keyword_set(column), $
                  /base_align_center, $
                  xpad=xpad,ypad=ypad,space=space)

;; label:
if n_elements(label) eq 1 then  $
  label=widget_label(base1,value=label,font=font,/align_center) 

;; position menu base:
base2=widget_base(base1, $
                  /row, $
                  /base_align_center)

;; position 'button':
s.position=widget_draw(base2, $
                       /button_events, $
                       xsize=xpsize,ysize=ypsize, $
                       tooltip='Position', $
                       uvalue=value[0])

;; context menu base:
s.position_menu=widget_base(s.position,/context_menu)

if keyword_set(row) and 0 then  $
  base1=widget_base(base1, $
                    /column,/base_align_left, $
                    xpad=xpad,ypad=0,space=0)

;; nobox
s.nobox= $
  cw_bgroup_rxo(base1,/nonexclusive, $
                space=0,xpad=0,ypad=0, $
                [' '], $
                tooltip='Draw Box', $
                uvalue='nobox')


;; boxfill menu base:
base2=widget_base(base1, $
                  /row, $
                  ypad=0, $
                  /base_align_center)

;; boxfill 'button':
s.boxfill=widget_draw(base2, $
                      /button_events, $
                      xsize=xsize,ysize=ysize, $
                      tooltip='Box Fill Color', $
                      uvalue=value[2])

;; context menu base:
s.boxfill_menu=widget_base(s.boxfill,/context_menu)

;; save the state into first child:
widget_control,widget_info(base,/child),set_uvalue=s

return,base
end
