;+
; NAME:
; 
;       CW_PLOTAXIS
;
; PURPOSE:
; 
;       A compound widget used to change the type, range and style
;       (bit 0 only) values of plot axis structure variable.
;      
; CATEGORY:
; 
;	Compound widgets.
;
; CALLING SEQUENCE:
; 
;	Result = CW_PLOTAXIS(PARENT,LABEL)
;
; INPUTS:
; 
;       PARENT - The ID of the parent widget.  
;
;       LABEL - a label to be drawn to the left of the widget.
;
; OPTIONAL KEYWORD PARAMETERS:
; 
;	UVALUE - Supplies the user value for the widget.
; 
;       FRAME - set to draw a frame around the widget.
;
;       VALUE - a 4-element array of the form [type,min,max,style].
;               type and min,max correspond to the !axis.type and
;               !axis.range variables, and style is bit 0 of
;               !axis.style.
;
;       FONT - font keyword for labels etc.
;       
;       ROW - set to orient the subwidgets in a row (default.)
;
;       COLUMN - set to orient the subwidgets in a column.
;
;       NO_RETURN - The default behavior is that the user must press
;                   <return> after entering new values.  Set this
;                   keyword so that new values are accepted even if
;                   the user just changes a value and then moves the
;                   cursor outside of the text entry area.
;
;
;       XPAD, YPAD, SPACE - keywords to widget_base
;
;       GRID - Set this to also add widgets to set the grid (i.e.,
;              ticklen) and gridstyle values.
;
;       NOGRID - Set this to draw the grid widgets but never map
;                them. Useful when aligning multiple cw_plotaxis
;                widgets.
;       
; OUTPUTS:
; 
;       The ID of the created widget is returned.
;
; COMMON BLOCKS:
;
;       CW_PLOTSTYLE: private common block containing color bitmaps
;       for 'buttons' and menus, and arrays of valid values for thick,
;       psym and symsize.
;
; RESTRICTIONS:
;
;       Uses the cgImage command in place of the tv command to display
;       images correctly on all devices. cgImage is part of the Coyote
;       Graphics System at www.idlcoyote.com.
;
; PROCEDURE/EXAMPLE:
;
;       The idea is that one or more instances of this cw would be
;       used in a widget intended to allow the user to interactively
;       adjust the settings for a plot.  For instance, you might have
;       a menu item such as Plot Options->Scaling, which would create
;       a popup widget containing CW_PLOTAXIS subwidgets for the X and
;       Y plot variables.  When the user makes changes to the Type,
;       Range, and Style values, the popup widget event handler would
;       re-draw the plot accordingly.
;
;       The widget returns events when any of it's children generate
;       events.  The returned event has the form
;       {CW_PLOTAXIS_EVENT,ID:id,TOP:top,HANDLER:handler,TAG:tag}
;       where TAG indicates which child widget generated the event:
;       possible values for EVENT.TAG are TYPE, MIN, MAX, and STYLE.
;       If GRID is set, then GRID and GRIDSTYLE tags are possible as
;       well.
;
; MODIFICATION HISTORY:
; 
;       David L. Windt, Bell Labs, March 1997
;       windt@bell-labs.com
;
;       DLW, June 1997, Added NO_RETURN keyword.
;
;       DLW, November 1997, Text fields for Range values are now
;       updated when the user makes a change; the specified values are
;       converted to floating point.
;
;       January 2004: Added XPAD, YPAD, SPACE keywords; Changed widget
;       types.  Added GRID keyword. Changed to double precision.
;
;       May 2013: Added color bitmaps for gridstyle options.
;
;-

;------------------------------------------------------------------
; event handler
function cw_plotaxis_ev,event
compile_opt hidden

common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

if tag_names(event,/str) eq 'WIDGET_DRAW' then begin

    ;; display context menu:
    widget_displaycontextmenu, $
      event.id,event.x,event.y,widget_info(event.id,/child)

    ;; set eventval:
    eventval=''

    ;; finished:
    goto,done
endif

;; get uvalue:
widget_control,event.id,get_uvalue=uvalue

;; set eventval:
eventval=uvalue

;; set the widget values of text widgets:
if (uvalue eq 'min') or (uvalue eq 'max') then begin
    ;; handle string conversion errors:
    on_ioerror,invalid_minmax
    valid=0
    
    ;; get value:
    widget_control,event.id,get_value=val

    ;; set value:
    val=double(val[0])
    valid=1

    ;; invalid entries:
    invalid_minmax: if valid eq 0 then val=0.d
    
    ;; update widget:
    widget_control,event.id,set_value=double(val)

endif

;; retrieve the state, which contains the widget id's
widget_control,widget_info(event.handler,/child),get_uvalue=s

;; grid button or gridstyle menu event:
if widget_info(s.grid,/valid) then $
  if strmid(uvalue,0,4) eq 'grid' then begin
    if uvalue eq 'grid' then begin
        ;; grid button:

        ;; sensitize gridstyle button:
        widget_control,s.gridstyle,map=event.select

    endif else begin
        ;; gridstyle menu button:

        ;; get linestyle index:
        gridstyle=strmid(uvalue,10)

        ;; reset eventval:
        eventval='gridstyle'

        ;; display the gridstyle:
        widget_control,s.gridstyle, $
          set_uvalue=gridstyle,get_value=wid

        if (!d.flags and 256) eq 256 then $
          if wid gt 0 then begin
            wset,wid
            cgimage,linestyle_bm[*,*,*,gridstyle]
        endif
    endelse
endif

done:

return,{CW_PLOTAXIS_EVENT,ID:event.handler,TOP:event.top, $
        HANDLER:0L,TAG:eventval}
end

;------------------------------------------------------------------
; set value procedure
pro cw_plotaxis_set_value,id,value
compile_opt hidden
on_error,2

common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; set the widget values:
widget_control,s.type,set_value=value[0]
widget_control,s.min,set_value=value[1]
widget_control,s.max,set_value=value[2]
widget_control,s.style,set_value=value[3]

if widget_info(s.grid,/valid) and (n_elements(value) eq 6) then begin
    widget_control,s.grid, $
      set_value=value[4] gt 0., $
      map=s.nogrid eq 0

    widget_control,s.gridstyle, $
      set_uvalue=value[5], $
      map=value[4] gt 0.

    widget_control,s.gridstyle,get_value=wid
    if (!d.flags and 256) eq 256 then $
      if wid gt 0 then begin
        wset,wid
        cgimage,linestyle_bm[*,*,*,value[5]]
    endif
endif

return
end

;------------------------------------------------------------------
; get value function
function cw_plotaxis_get_value,id
compile_opt hidden
on_error,2

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; get the widget values:
widget_control,s.type,get_value=type
widget_control,s.min,get_value=min
widget_control,s.max,get_value=max
widget_control,s.style,get_value=style
value=[double(type),double(min),double(max),double(style)]
if widget_info(s.grid,/valid) then begin
    widget_control,s.grid,get_value=grid
    widget_control,s.gridstyle,get_uvalue=gridstyle
    value=[value,grid ? .5 : 0.,double(gridstyle)]
endif 
return,value
end

;------------------------------------------------------------------
; main function
function cw_plotaxis,parent,label,uvalue=uvalue,frame=frame,value=value, $
           font=font,row=row,column=column,no_return=no_return, $
           xpad=xpad,ypad=ypad,space=space, $
           grid=grid,nogrid=nogrid

on_error,0
common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

if n_params() eq 0 then message,'Must specify a parent for cw_plotaxis.'
if not keyword_set(uvalue) then uvalue=0
if (keyword_set(column) eq 0) then row=1

;; check that value is set properly
if keyword_set(grid) eq 0 then begin
    if n_elements(value) ne 4 then value=dblarr(4)
endif else begin
    if n_elements(value) ne 6 then value=dblarr(6)
endelse

;; keep widget id's in state:
s={type:0L,min:0L,max:0L,style:0L,grid:0L, $
   gridstyle:0L,gridstyle_menu:0L, $
   nogrid:keyword_set(nogrid)}

if n_elements(font) eq 0 then font=''

base=widget_base(parent,uvalue=uvalue,frame=keyword_set(frame), $
                 event_func='cw_plotaxis_ev', $
                 pro_set_value='cw_plotaxis_set_value', $
                 func_get_value='cw_plotaxis_get_value', $
                 xpad=xpad,ypad=ypad,space=space,/base_align_right) 

base1=widget_base(base,row=keyword_set(row),column=keyword_set(column), $
                  xpad=xpad,ypad=ypad,space=space, $
                  /base_align_center)

if n_elements(label) eq 1 then $
  label=widget_label(base1,value=label,font=font,/sunken_frame) 

s.min=cw_field_rxo(base1,title='Min:',/return_events, $
                   value=value(1),uvalue='min',xsize=12, $
                   font=font,no_return=keyword_set(no_return))
s.max=cw_field_rxo(base1,title='Max:',/return_events, $
                   value=value(2),uvalue='max',xsize=12, $
                   font=font,no_return=keyword_set(no_return))

s.style=cw_bgroup_rxo(base1,/nonexclusive, $
                      space=space,xpad=xpad,ypad=ypad, $
                      ['Exact'],uvalue='style',font=font)
widget_control,s.style,set_value=value[3]

s.type=cw_bgroup_rxo(base1,/nonexclusive, $
                     space=space,xpad=xpad,ypad=ypad, $
                     ['Log'],uvalue='type',font=font)
widget_control,s.type,set_value=value[0]

if keyword_set(grid) then begin
    s.grid=cw_bgroup_rxo(widget_base(base1,space=0,xpad=0,ypad=0), $
                         /nonexclusive, $
                         space=space,xpad=xpad,ypad=ypad, $
                         ['Grid'],uvalue='grid',font=font)
    widget_control,s.grid,set_value=value[4]

    ;; rely on cw_plotstyle to make linestyle button bitmaps:
    n_linestyle: sz=size(linestyle_bm)
    n_linestyle=sz[0] eq 0 ? 0 : sz[4]
    if n_linestyle eq 0 then begin
        c=cw_plotstyle(/init_only)
        goto,n_linestyle
    endif

    base2=widget_base(base1,/row,space=0,xpad=0,ypad=0,/base_align_center)

    ;; get linestyle_bm image size:
    xsize=sz[1]
    ysize=sz[2]

    ;; gridstyle 'button':
    s.gridstyle=widget_draw(base2, $
                            /button_events, $
                            xsize=xsize,ysize=ysize, $
                            tooltip='Grid Linestyle', $
                            uvalue=value[5])

    ;; gridstyle contextmenu base:
    s.gridstyle_menu=widget_base(s.gridstyle,/context_menu)

    for i=0,5 do $
      button=widget_button(s.gridstyle_menu, $
                           value=linestyle_bm[*,*,*,i], $
                           uvalue='gridstyle_'+strtrim(i,2))

    ;; set initial value:
    ;; gridstyle is insensitive if grid=0
    widget_control,s.gridstyle,map=value[4] eq 1
endif

;; stuff the state into first child (i.e., base1)
widget_control,widget_info(base,/child),set_uvalue=s

return,base
end
