;+
; NAME:
; 
;       CW_PLOTSTYLE
;
; PURPOSE:
; 
;       A compound widget used to set values for the graphics keywords
;       COLOR, LINESTYLE, THICK, PSYM, and SYMSIZE.
;      
; CATEGORY:
; 
;	Compound widgets.
;
; CALLING SEQUENCE:
; 
;	Result = CW_PLOTSTYLE(PARENT,LABEL)
;
; INPUTS:
; 
;       PARENT - The ID of the parent widget.  
;
;       LABEL - a label to be drawn to the left (or top, for /column)
;               of the widget.
;
; OPTIONAL KEYWORD PARAMETERS:
; 
;	UVALUE - Supplies the user value for the widget.
;
;       VALUE - an array of initial values:
;               [COLOR,LINESTYLE,THICK,PSYM,SYMSIZE]
;
;       FRAME - set to draw a frame around the widget.
;       
;       FONT - font keyword for labels etc.
;
;       ROW - set to orient the subwidgets in a row (default.)
;
;       COLUMN - set to orient the subwidgets in a column.
;
;       NO_PSYM - set to omit the PSYM and SYMSIZE widgets. If NO_SYM
;                 is set, then PSYM=0 and SYMSIZE=0 will be returned
;                 when using WIDGET_CONTROL,GET_VALUE; PSYM and
;                 SYMSIZE are ignored when using
;                 WIDGET_CONTROL,SET_VALUE
;
;       XPAD, YPAD, SPACE - keywords to widget_base
;
;       INIT_ONLY - set to create the 24-bit bitmaps and value arrays
;                   that are stored in the private CW_PLOTSTYLE common
;                   block listed below.
;
; OUTPUTS:
; 
;       The ID of the created widget is returned.
;
; COMMON BLOCKS:
;
;       CW_PLOTSTYLE: private common block containing color bitmaps
;       for 'buttons' and menus, and arrays of valid values for thick,
;       psym and symsize.
;
; RESTRICTIONS:
;
;       Uses the cgImage command in place of the tv command to display
;       images correctly on all devices. cgImage is part of the Coyote
;       Graphics System at www.idlcoyote.com.
;
; PROCEDURE/EXAMPLE:
;
;       The idea is that one or more instances of this cw would be
;       used in a widget intended to allow the user to interactively
;       adjust the settings for a plot.  For instance, you might have
;       a menu item such as Plot Options->Styles, which would create a
;       popup widget containing CW_PLOTSTYLE subwidgets for each of
;       the variables being plotted.  When the user makes changes to
;       the Color, Linestyle, Thick, Psym, and Symsize values, the
;       popup widget event handler would re-draw the plot accordingly.
; 
;       The user is presented with pulldown menus displaying:
;
;       - 32 color choices corresponding to the 1st 32 color table
;         entries.
;
;       - 6 linestyle choices (linestyle=0 to 5)
;
;       - 9 thickness choices (thick=1 to 9)
;
;       - 35 psym choices, corresponding to the 18 symbols obtained
;         using the SYM function with and without lines.  Note that
;         the value of psym returned by this widget is intended
;         therefore to be used with the SYM function.
;
;       - 8 symsize choices (symsize=0.25 to 2.00, in 0.25 increments)
;
;       The widget returns events when any of it's children generate
;       events.  The returned event has the form
;       {CW_PLOTSTYLE_EVENT,ID:id,TOP:top,HANDLER:handler,TAG:tag}
;       where TAG indicates which child widget generated the event:
;       Possible values for EVENT.TAG are COLOR, LINESTYLE, THICK
;       PSYM, and SYSMSIZE.
;
; MODIFICATION HISTORY:
; 
;       David L. Windt, Bell Labs, March 1997
;       windt@bell-labs.com
;
;       January 2004: Added XPAD, YPAD, SPACE keywords, and added draw
;       widgets to display results.
;
;       May 2013: Re-written to use 24-bit color bitmap buttons and
;       pull-down menus in place of draw and droplist widgets.  No
;       longer reliant on using TEK_COLOR; load whatever colors you
;       like in the first 32 color indices of the color table.
;
;       DLW, RXO, davidwindt@gmail.com.
;
; -

;------------------------------------------------------------------
; event handler
function cw_plotstyle_ev,event

compile_opt hidden

on_error,2

common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

;; retrieve the state, which contains the widget id's
widget_control,widget_info(event.handler,/child),get_uvalue=s

if tag_names(event,/str) eq 'WIDGET_DRAW' then begin

    ;; menu base:
    menu_base=widget_info(event.id,/child)

    ;; create context menu if it doesn't exist:
    if widget_info(menu_base,/child) eq 0L then begin

        ;; which menu:
        case event.id of
            s.color: begin
                bm=color_bm
                uvalue_base='color_'
            end
            s.linestyle: begin
                bm=linestyle_bm
                uvalue_base='linestyle_'
            end
            s.thick: begin
                bm=thick_bm
                uvalue_base='thick_'
            end
            s.psym: begin
                bm=psym_bm
                uvalue_base='psym_'
            end
            s.symsize: begin
                bm=symsize_bm
                uvalue_base='symsize_'
            end
        endcase

        ;; how many:
        n=(size(bm))[4]

        ;; make a button for each::
        for j=0,n-1 do $
          button=widget_button(menu_base, $
                               value=bm[*,*,*,j], $
                               uvalue=uvalue_base+strtrim(j,2))
    endif

    ;; display context menu:
    widget_displaycontextmenu, $
      event.id,event.x,event.y,menu_base

    return,0

endif

;; get eventval:
widget_control,event.id,get_uvalue=uvalue

;; get value:
if n_elements(uvalue) eq 1 then begin

    case 1 of
        strmid(uvalue,0,6) eq 'color_': begin
            ;; get color index:
            color=(fix(strmid(uvalue,6)) < ((size(color_bm))[4]-1)) > 0
            widget_control,s.color, $
              get_value=wid, $
              set_uvalue=color

            eventval=color
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,color_bm[*,*,*,color]
            endif
        end
        strmid(uvalue,0,10) eq 'linestyle_': begin
            ;; get linestyle index:
            linestyle=(fix(strmid(uvalue,10)) < ((size(linestyle_bm))[4]-1)) > 0
            widget_control,s.linestyle, $
              get_value=wid, $
              set_uvalue=linestyle

            eventval=linestyle
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,linestyle_bm[*,*,*,linestyle]
            endif
        end
        strmid(uvalue,0,6) eq 'thick_': begin
            ;; get thick value
            thick_index=(float(strmid(uvalue,6)) < ((size(thick_bm))[4]-1)) > 0
            thick=thick_values[thick_index]
            widget_control,s.thick, $
              get_value=wid, $
              set_uvalue=thick

            eventval=thick
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,thick_bm[*,*,*,thick_index]
            endif
        end
        strmid(uvalue,0,5) eq 'psym_': begin
            ;; get psym value:
            psym_index=(float(strmid(uvalue,5)) < ((size(psym_bm))[4]-1)) > 0
            psym=psym_values[psym_index]
            widget_control,s.psym, $
              get_value=wid, $
              set_uvalue=psym

            ;; linestyle is insensitive if psym le 0:
            if widget_info(s.psym,/valid) then $
              widget_control,s.linestyle,map=(psym le 0)

            ;; symsize is only sensitive if psym ne 0:
            if widget_info(s.symsize,/valid) then $
              widget_control,s.symsize,map=(psym ne 0)

            eventval=psym
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,psym_bm[*,*,*,psym_index]
            endif
        end
        strmid(uvalue,0,8) eq 'symsize_': begin
            ;; get symsize value:
            symsize_index=(float(strmid(uvalue,8)) < ((size(symsize_bm))[4]-1)) > 0
            symsize=symsize_values[symsize_index]
            widget_control,s.symsize, $
              get_value=wid, $
              set_uvalue=symsize

            eventval=symsize
            if (!d.flags and 256) eq 256 then $
              if wid gt 0 then begin
                wset,wid
                cgimage,symsize_bm[*,*,*,symsize_index]
            endif
        end
        else: eventval=0
    endcase
endif

done: ;;

return,{CW_PLOTSTYLE_EVENT,ID:event.handler,TOP:event.top,HANDLER:0L,TAG:eventval}
end

;------------------------------------------------------------------
; set value procedure
pro cw_plotstyle_set_value,id,value

compile_opt hidden

on_error,2

common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; make sure values are in range:
color=(value[0] < ((size(color_bm))[4]-1)) > 0
linestyle=(value[1] < ((size(linestyle_bm))[4]-1)) > 0
thick=(value[2] < ((size(thick_bm))[4]-1)) > 0
psym=(value[3] < max(psym_values)) > min(psym_values)
symsize=(value[4] < max(symsize_values)) > min(symsize_values)

;; set the widget values:

;; color:
widget_control,s.color,set_uvalue=color,get_value=wid
if (!d.flags and 256) eq 256 then $
  if wid gt 0 then begin
    wset,wid
    cgimage,color_bm[*,*,*,color]
endif

;; linestyle:
widget_control,s.linestyle, $
  set_uvalue=linestyle,map=(psym le 0),get_value=wid
if (!d.flags and 256) eq 256 then $
  if wid gt 0 then begin
    wset,wid
    cgimage,linestyle_bm[*,*,*,linestyle]
endif

;; thickness:
thick_index=where(thick_values eq thick,count)
if count eq 0 then thick_index=0
widget_control,s.thick, $
  set_uvalue=thick,get_value=wid
if (!d.flags and 256) eq 256 then $
  if wid gt 0 then begin
    wset,wid
    cgimage,thick_bm[*,*,*,thick_index]
endif

;; psym and symsize:
if widget_info(s.psym,/valid) then begin

    ;; psym:
    psym_index=where(psym_values eq psym,count)
    if count eq 0 then psym_index=0
    widget_control,s.psym, $
      set_uvalue=psym_values[psym_index],get_value=wid
    if (!d.flags and 256) eq 256 then $
      if wid gt 0 then begin
        wset,wid
        cgimage,psym_bm[*,*,*,psym_index]
    endif

    ;; symsize:
    symsize_index=where(symsize_values eq symsize,count)
    if count eq 0 then symsize_index=3
    widget_control,s.symsize, $
      set_uvalue=symsize_values[symsize_index],get_value=wid, $
      map=(psym ne 0)
    if (!d.flags and 256) eq 256 then $
      if wid gt 0 then begin
        wset,wid
        cgimage,symsize_bm[*,*,*,symsize_index]
    endif

endif

return
end

;------------------------------------------------------------------
; get value function
function cw_plotstyle_get_value,id

compile_opt hidden
on_error,2

;; retrieve the state, which contains the widget id's
widget_control,widget_info(id,/child),get_uvalue=s

;; get the widget values:

;; color:
widget_control,s.color,get_uvalue=color

;; linestyle:
widget_control,s.linestyle,get_uvalue=linestyle

;; thickness:
widget_control,s.thick,get_uvalue=thick

;; psym and symsize:
if widget_info(s.psym,/valid) then begin
    widget_control,s.psym,get_uvalue=psym
    widget_control,s.symsize,get_uvalue=symsize
endif else begin
    psym=0
    symsize=0
endelse

return,[color,linestyle,thick,psym,symsize]
end

;------------------------------------------------------------------
; main function
function cw_plotstyle,parent,label,uvalue=uvalue, $
           frame=frame,value=value,font=font,row=row,column=column, $
           no_psym=no_psym,xpad=xpad,ypad=ypad,space=space, $
           init_only=init_only

on_error,2

common cw_plotstyle,color_bm,linestyle_bm,thick_bm,psym_bm,symsize_bm, $
  thick_values,psym_values,symsize_values

;; bitmap size:
xsize=40
ysize=10

;; default values:
n_color=32
n_linestyle=6
thick_values=1.+findgen(19)/2.
n_thick=n_elements(thick_values)
psym_values=[indgen(18),-(indgen(17)+1)]
n_psym=n_elements(psym_values)
symsize_values=[.1,.2,.3,.4,.5,.6,.7,.8,.9,1.0,1.25,1.50,1.75,2.00]
n_symsize=n_elements(symsize_values)

;; create bitmaps if undefined, or init_only is set:
sz=size(color_bm)
init=(sz[0] eq 0) ? 1 : (sz[4] ne n_color)
if init or keyword_set(init_only) then begin
    ;; get current color table:
    tvlct,r,g,b,/get

    ;; colors (loaded from current color table):
    color_bm=bytarr(xsize,ysize,3,n_color)
    for j=0,n_color-1 do begin
        color_bm[*,*,0,j]=r[j]
        color_bm[*,*,1,j]=g[j]
        color_bm[*,*,2,j]=b[j]
    endfor

    ;; make pixmap for creating remaining bitmaps:
    window,/pixmap,/free,xsize=xsize,ysize=ysize

    ;; save a copy of the current color table:
    r_save=r
    g_save=g
    b_save=b

    ;; create black and white in first to color indices:
    r[0]=0
    g[0]=0
    b[0]=0
    r[1]=255
    g[1]=255
    b[1]=255

    ;; load new color table
    tvlct,r,g,b

    ;; linestyles:
    linestyle_bm=bytarr(xsize,ysize,3,n_linestyle)
    for j=0,n_linestyle-1 do begin
        erase,color=1
        plots,[0,1],[.5,.5],/norm,color=0,linestyle=j,thick=1,psym=0
        linestyle_bm[*,*,*,j]=tvrd(true=3)
    endfor

    ;; thick:
    thick_bm=bytarr(xsize,ysize,3,n_thick)
    for j=0,n_thick-1 do begin
        erase,color=1
        plots,[0,1],[.5,.5],/norm,color=0,linestyle=0,thick=thick_values[j],psym=0
        thick_bm[*,*,*,j]=tvrd(true=3)
    endfor

    ;; create vectors for plots:
    xx=[.1,.5,.9]
    yy=xx*0+.5

    ;; psym:
    psym_bm=bytarr(xsize,ysize,3,n_psym)
    for j=0,n_psym-1 do begin
        erase,color=1
        plots,xx,yy,/norm,color=0,linestyle=0,thick=1,psym=sym(psym_values[j]),symsize=1
        psym_bm[*,*,*,j]=tvrd(true=3)
    endfor

    ;; symsize:
    symsize_bm=bytarr(xsize,ysize,3,n_symsize)
    for j=0,n_symsize-1 do begin
        erase,color=1
        plots,xx,yy,/norm,color=0,linestyle=0,thick=1,psym=sym(5),symsize=symsize_values[j]
        symsize_bm[*,*,*,j]=tvrd(true=3)
    endfor

    ;; delete pixmap:
    wdelete

    ;; restore old color table
    tvlct,r_save,g_save,b_save
endif

if keyword_set(init_only) then return,1

if n_params() eq 0 then message,'Must specify a parent for cw_plotstyle.'
if not keyword_set(uvalue) then uvalue=0
if keyword_set(column) eq 0 then row=1

;; check that value is set properly
if n_elements(value) ne 5 then value=[1,1,1,0,1]

value=[(value[0] < (n_color-1)) > 0, $
       (value[1] < (n_linestyle-1)) > 0, $
       (value[2] < (n_thick-1)) > 0, $
       (value[3] < max(psym_values)) > min(psym_values) , $
       (value[4] < max(symsize_values)) > min(symsize_values)] 

; keep widget id's in state:
s={color:0L, $
   color_menu:0L, $
   linestyle:0L, $
   linestyle_menu:0L, $
   thick:0L, $
   thick_menu:0L, $   
   psym:0L, $
   psym_menu:0L, $
   symsize:0L, $
   symsize_menu:0L}

if n_elements(font) ne 1 then font=''

;; base:
base=widget_base(parent, $
                 uvalue=uvalue, $
                 frame=keyword_set(frame), $
                 event_func='cw_plotstyle_ev', $
                 pro_set_value='cw_plotstyle_set_value', $
                 func_get_value='cw_plotstyle_get_value', $
                 xpad=xpad,ypad=ypad,space=space) 

base1=widget_base(base, $
                  row=keyword_set(row), $
                  column=keyword_set(column), $
                  /base_align_center, $
                  xpad=xpad,ypad=ypad,space=space)

;; label:
if n_elements(label) eq 1 then  $
  label=widget_label(base1,value=label,font=font,/align_center) 

;; color menu base:
base2=widget_base(base1, $
                  /row, $
                  /base_align_center)

;; color 'button'
s.color=widget_draw(base2, $
                    /button_events, $
                    xsize=xsize,ysize=ysize, $
                    tooltip='Color', $
                    uvalue=value[0])

;; context menu base:
s.color_menu=widget_base(s.color,/context_menu)

;; linestyle menu base:
base2=widget_base(base1, $
                  /row, $
                  /base_align_center)

;; linestyle 'button'
s.linestyle=widget_draw(base2, $
                        /button_events, $
                        xsize=xsize,ysize=ysize, $
                        tooltip='Linestyle', $
                        uvalue=value[1])

;; linestyle is insensitive if psym gt 0:
widget_control,s.linestyle,map=(value[3] le 0)

;; context menu base:
s.linestyle_menu=widget_base(s.linestyle,/context_menu)

;; thick menu base:
base2=widget_base(base1, $
                  /row, $
                  /base_align_center)


;; thick 'button'
s.thick=widget_draw(base2, $
                    /button_events, $
                    xsize=xsize,ysize=ysize, $
                    tooltip='Thickness', $
                    uvalue=value[2])

;; thick index:
thick_index=where(thick_values eq value[3],count)
if count eq 0 then thick_index=0

;; context menu base:
s.thick_menu=widget_base(s.thick,/context_menu)

if keyword_set(no_psym) eq 0 then begin
    ;; psym menu base:
    base2=widget_base(base1, $
                      /row, $
                      /base_align_center)

    ;; psym index:
    psym_index=where(psym_values eq value[3],count)
    if count eq 0 then psym_index=0

    ;; psym 'button'
    s.psym=widget_draw(base2, $
                       /button_events, $
                       xsize=xsize,ysize=ysize, $
                       tooltip='Symbol', $
                       uvalue=value[3])

    ;; context menu base:
    s.psym_menu=widget_base(s.psym,/context_menu)

    ;; symsize menu base:
    base2=widget_base(base1, $
                      /row, $
                      /base_align_center)

    ;; symsize index:
    symsize_index=where(symsize_values eq value[4],count)
    if count eq 0 then symsize_index=0

    ;; symsize 'button'
    s.symsize=widget_draw(base2, $
                          /button_events, $
                          xsize=xsize,ysize=ysize, $
                          tooltip='Symbol Size', $
                          uvalue=value[4])

    ;; symsize is only sensitive if psym ne 0
    widget_control,s.symsize,map=(value[3] ne 0)

    ;; context menu base:
    s.symsize_menu=widget_base(s.symsize,/context_menu)

endif

;; save the state into first child:
widget_control,widget_info(base,/child),set_uvalue=s
return,base
end
