;
;+
; NAME:
; 
;	DIALOG
;
; PURPOSE:
; 
;       A popup widget dialog box to get user input. Like
;       WIDGET_MESSAGE, which is better in some cases, but this widget
;       also includes fields and lists.
;
; CATEGORY:
; 
;	Widgets.
;
; CALLING SEQUENCE:
;
;	Result = DIALOG([TEXT])
;
; OPTIONAL INPUTS:
; 
;	TEXT - The label seen by the user.
;	
; KEYWORD PARAMETERS:
; 
;   There are 6 types of dialogs, each with unique behavior.  With
;   each default dialog type are associated buttons; these buttons can
;   be overridden with the BUTTONS keyword, except in the case of the
;   LIST and FIELD dialogs.
;
;       One of the following six keywords MUST be set:
;
;       ERROR - Display an error message; default BUTTONS =
;               ['Abort','Continue']
;
;       WARNING - Display a warning message.  default BUTTONS = ['OK']
;
;       INFO - Display an informational message; 
;              default BUTTONS = ['Cancel','OK']
; 
;       QUESTION - Ask a question.  default BUTTONS =
;                  ['Cancel','No','Yes']
;
;       LIST - Get a selection from a list of choices.  default
;              BUTTONS = ['Cancel','OK'] Must specify CHOICES = string
;              array of list choices.
;
;              Set the RETURN_INDEX keyword to cause the returned
;	       value to be the zero-based index of the selected
;	       list item.
;
;              FILTER can be set to allow the user to filter the list
;              of choices. When FILTER is set, the CHOICE_TYPES keyword
;              must also be supplied: CHOICE_TYPES = string array of
;              names, same length as the CHOICE array, that indicates
;              the type for each element of choice.
;
;              FILTER example:
;
;                FILTER=1
;              
;                CHOICES=['Red','Blue','Green','One','Two','Three']
;
;                CHOICE_TYPES=['Colors','Colors','Colors','Numbers','Numbers','Numbers'].
;	       
;	         When the user click the "Filter" button that will be
;                displayed, ; only Colors or Numbers will be listed,
;                depending on ; which type is selected from the
;                displayed droplist of choice types.
;
;       FIELD - Get user input, using CW_FIELD.  default BUTTONS =
;               ['Cancel','OK']. FLOAT, INTEGER, LONG, and STRING
;               keywords apply here, as does the VALUE keyword to set
;               an initial value.  Furthermore, the TEXT input
;               variable can be specified as a two-element array, in
;               which case the 2nd element will appear AFTER the
;               FIELD.
;
;   XSIZE - X-Size of FIELD
;
;   GROUP - Group leader keyword.
;
;   TITLE - title of popup widget.
;
; OUTPUTS:
; 
;       In the case of LIST or FIELD dialogs, this function returns
;       the selected list element or the user input, respectively.
;       Otherwise, this function returns the name of the pressed
;       button.
;
; EXAMPLE:
; 
;	1. Create a QUESTION DIALOG widget.
;
;       D = DIALOG(/QUESTION,'Do you want to continue?')
;
;       2. Get the user to enter a number.
;
;       D = DIALOG(/FLOAT,VALUE=3.14159,'Enter a new value for pi.')
; 
;       3. Get the user to choose from a list of options.
;
;       D = DIALOG(/LIST,CHOICES=['Snoop','Doggy','Dog'])
;
; MODIFICATION HISTORY:
; 
;       David L. Windt, Bell Labs, March 1997
;
;       May 1997 - Added GROUP keyword, and modified use of MODAL
;                  keyword to work with changes in IDL V5.0
;
;       Feb 2013 - Added 2-element TEXT parameter and XSIZE keyword for FIELD dialogs
;
;       May 2013 - Added FILTER and CHOICE_TYPES keywords; removed
;                  common block.
;                  
;       davidwindt@gmail.com
;-

pro dialog_event,event

on_error,2

;; get state:
widget_control,event.top,get_uvalue=s_ptr
s=*s_ptr

;; extract pars from state:
field_id=s.field_id
choices=s.choices
list_id=s.list_id
listindex=s.listindex

;; widget resize?
if (tag_names(event,/structure_name) eq 'WIDGET_BASE') then begin
    old_sz=s.sz

    ;; get the new widget size:
    widget_control,event.top,tlb_get_size=sz

    ;; compute difference:
    d_x=sz[0]-old_sz[0]
    d_y=sz[1]-old_sz[1]

    ;; save new size:
    s.sz=sz
    *s_ptr=s

    ;; get current list size:
    g=widget_info(list_id,/geometry)

    ;; set new list widget size:
    widget_control,list_id, $
      scr_xsize=(g.scr_xsize+d_x)>1, $
      scr_ysize=(g.scr_ysize+d_y)>1

    return
endif

; get uvalue of this event
widget_control,event.id,get_uvalue=uvalue

case uvalue of 

    'list': begin
        ;; define selection:
        *s.selection=listindex ? event.index : choices[event.index]

        ;; if filtered list, then return index from original list:
        if listindex and s.filter then $
          *s.selection=(*s.filtered_choice_indices)[*s.selection]

        ;; update state:
        *s_ptr=s

        ;; if user double-clicks, then we're done:
        if event.clicks ne 2 then return
    end

    'field': begin
        ;; get selection:
        widget_control,s.field_id,get_value=selection

        ;; save it:
        *s.selection=selection

        ;; update state:
        *s_ptr=s

        return
    end

    'filter_type': begin
        
        ;; set flag to update list:
        filter_list=1
    end

    'buttons': begin
        case 1 of

            ;; field widget?
            widget_info(s.field_id,/valid): begin
                if (event.value eq 'Cancel') then  $
                  selection=event.value else  $
                  widget_control,s.field_id,get_value=selection

                ;; define selection:
                *s.selection=selection

                ;; save it:
                *s_ptr=s
            end

            ;; list widget?
            widget_info(list_id,/valid): begin
                ;; define selection:
                if (event.value eq 'Cancel') then (*s.selection=listindex ? -1 : event.value) $
                else begin
                    id=widget_info(list_id,/list_select)
                    *s.selection=listindex ? id : ((id ge 0) ? choices[id] : 'Cancel')

                    ;; if filtered list, then return index from original list:
                    if listindex and s.filter then $
                      *s.selection=(*s.filtered_choice_indices)[*s.selection]
                endelse

                ;; save it:
                *s_ptr=s
            end

            else: begin
                ;; get selection:
                *s.selection=event.value

                ;; save it:
                *s_ptr=s
            end

        endcase
    end
endcase

;; filtered list?
if keyword_set(filter_list) then begin

    ;; what's selected on the droplist:
    filter_type=widget_info(s.filter_type_id,/droplist_select)

    ;; filter?
    s.filter=filter_type ne 0

    if s.filter eq 0 then begin
        ;; unfiltered:

        ;; offer all choices:
        widget_control,list_id,set_value=s.choices 

    endif else begin
        ;; offer filtered choices:

        ;; which item is selected:
        widget_control,s.filter_type_id,get_value=filtered_choices
        filtered_choice=filtered_choices[filter_type]

        ;; extract filtered choices:
        wh=where(filtered_choice eq s.choice_types,count)
        *s.filtered_choice_indices=wh
        *s.filtered_choices=s.choices[wh]

        ;; update state:
        *s_ptr=s

        ;; update list:
        widget_control,s.list_id,set_value=*s.filtered_choices

    endelse
    return
endif

;; we're done - destroy widget:
widget_control,event.top,/destroy

return
end

function dialog,text,buttons=buttons, $
                error=error,warning=warning,info=info,question=question, $
                field=field,float=float,integer=integer, $
                long=long,string=string,value=value, $
                list=list,choices=choices,return_index=return_index, $
                choice_types=choice_types,filter=filter, $
                title=title,group=group,xsize=xsize, $
                resource_name_base=resource_name_base, $
                resource_name_list=resource_name_list

on_error,2

if keyword_set(title) eq 0 then title=' '

; make widget base:
if keyword_set(group) eq 0 then group=0L
if (strmid(!version.release,0,1) ge '5') and $
  widget_info(long(group),/valid) then $
  base=widget_base(title=title,/column,/base_align_center,/modal,group=group, $
                   tlb_size_events=keyword_set(list), $
                   resource_name=n_elements(resource_name_base) eq 1 ? $
                   resource_name_base : '') $
else base=widget_base(title=title,/column,/base_align_center) 

if keyword_set(float) then field=1
if keyword_set(integer) then field=1
if keyword_set(long) then field=1
if keyword_set(string) then field=1
if n_elements(value) eq 0 then value=0
if n_elements(choices) gt 0 then list=1
if n_elements(choice_types) eq 0 then choice_types=-1

; widget configuration depends on type of dialog:
case 1 of
    keyword_set(error):begin
        if n_params() eq 0 then text='Error' else $
          text='Error: '+text
        label=widget_label(base,value=text,frame=0)
        if keyword_set(buttons) eq 0 then buttons=['Abort','Continue']
    end
    keyword_set(warning):begin
        if n_params() eq 0 then text='Warning' else $
          text='Warning: '+text
        label=widget_label(base,value=text,frame=0)
        if keyword_set(buttons) eq 0 then buttons=['OK']
    end
    keyword_set(info):begin
        if n_params() eq 0 then text=' '
        label=widget_label(base,value=text,frame=0)
        if keyword_set(buttons) eq 0 then buttons=['Cancel','OK']
    end
    keyword_set(question):begin
        if n_params() eq 0 then text='Question?' 
        label=widget_label(base,value=text,frame=0)
        if keyword_set(buttons) eq 0 then buttons=['Cancel','No','Yes']
    end
    keyword_set(field):begin
        isfield=1
        if n_params() eq 0 then text='Input: ' 
        sz=size(value)
        if keyword_set(string) and (sz(1) ne 7) then value=strtrim(value)
        field_id=cw_field_rxo(base,title=text[0], $
                              uvalue='field', $  
                              value=value, $
                              /return_events, $
                              /no_return, $
                              xsize=n_elements(xsize) eq 1 ? xsize : $
                              (n_elements(value) eq 1 ? strlen(value) : 6), $
                              units=n_elements(text) eq 2 ? text[1] : '', $
                              float=keyword_set(float), $
                              integer=keyword_set(integer), $
                              long=keyword_set(long), $
                              string=keyword_set(string))
        buttons=['Cancel','OK']
    end
    keyword_set(list):begin
        if keyword_set(choices) eq 0 then $
          message,'Must supply an array of choices for the list.'
        if n_params() eq 0 then text='Choose: '
        label=widget_label(base,value=text[0],frame=0)
        list_id=widget_list(base,value=choices, $
                            ysize=n_elements(choices) < 30, $
                            resource_name=n_elements(resource_name_list) eq 1 ? $
                            resource_name_list : '', $
                            uvalue='list')

        ;; filter?
        if keyword_set(filter) and (n_elements(choice_types) gt 1) then begin

            ;; sort choice_types and retain only unique items:
            s_choice_types=choice_types[sort(choice_types)]
            u_choice_types=s_choice_types[uniq(s_choice_types)]

            row=widget_base(base,/row,space=0,xpad=0,ypad=0)

            label=widget_label(row,value='Filter:')

            filter_type_id=widget_droplist(row, $
                                           value=['None', $
                                                  u_choice_types], $
                                           /dynamic_resize, $
                                           uvalue='filter_type')
        endif
        
        buttons=['Cancel','OK']
    end
endcase

; make buttons:
bgroup=cw_bgroup_rxo(base,/row,buttons,uvalue='buttons',/return_name)

; realize widget:
widget_control,base,/realize

;; get base size:
widget_control,base,tlb_get_size=sz

;; create state:
s={sz:sz, $
   selection:ptr_new(/allocate_heap), $
   field_id:n_elements(field_id) eq 0 ? 0L : field_id, $
   choices:n_elements(choices) eq 0 ? '' : choices, $
   choice_types:n_elements(choice_types) eq 0 ? '' : choice_types, $
   filter:0, $
   filter_type:'', $
   filter_type_id:n_elements(filter_type_id) eq 0 ? '' : filter_type_id, $
   filtered_choices:ptr_new(/allocate_heap), $
   filtered_choice_indices:ptr_new(/allocate_heap), $
   list_id:n_elements(list_id) eq 0 ? 0L: list_id, $
   listindex:keyword_set(return_index)}

;; save ptr to state base:
s_ptr=ptr_new(s)
widget_control,base,set_uvalue=s_ptr

; manage widget:
if (strmid(!version.release,0,1) ge '5') and widget_info(long(group),/valid) then $
  xmanager,'dialog',base,group=group else $
  xmanager,'dialog',base,/modal

;; get updated state:
s=*s_ptr

;; get selection:
selection=*s.selection

;; free pointers:
ptr_free,s.selection,s.filtered_choices,s.filtered_choice_indices

; return selection:
return,selection
end


