;+
; NAME:
; 
;	EDGE_FIND
;
; PURPOSE:
; 
;       Return the center of the rising or falling edge of the
;       supplied data array.
;
; CALLING SEQUENCE:
; 
;	Result = EDGE_FIND(X,Y[,/RISING][,/FALLING])
;
; INPUTS:
; 
;       X, Y - 1D data arrays.
;
; OUTPUTS:
; 
;       Result = The X value corresponding to the center of the rising
;       or falling edge of the Y data.
;
; KEYWORD PARAMETERS:
; 
;       RISING - Set this keyword to find the rising edge. This is the default.
;       
;       FALLING - Set this keyword to find the falling edge.
;
; EXAMPLE:
;
;       Make some noisy data:
;
;       x=VECTOR(-8.,8.,100)
;       y=ATAN(x)+.1*RANDOMN(seed,100)
;
;       Determine the rising edge:
;
;	x_edge=EDGE_FIND(y)
;
; PROCEDURE:
;
;       Pretty cheesy: locate the first maximum (rising edge) or
;       minimum (falling edge) of the derivative of Y.  There's almost
;       certainly a better way...
;
; MODIFICATION HISTORY:
; 
; 	David L. Windt, December 2003
; 	
;       windt@astro.columbia.edu
;                    
;-
function edge_find,x,y,rising=rising,falling=falling

;; compute derivative:
dy=deriv(x,y)

if keyword_set(falling) then begin
    ;; falling edge:
    y_edge=min(dy)
    x_edge=x[!c]
endif else begin
    ;; rising edge:
    y_edge=max(dy)
    x_edge=x[!c]
endelse

return,x_edge
end
