;+
; NAME:
;
;      EROM
;      
; PURPOSE:
;
;       Read columns of data from a text file.
;
;       This program can be used to read data written by the MORE
;       program.
;
;       The file to be read must be such that if the data are
;       space-separated, then all variables are numeric; String
;       variables are allowed only if the data are separated by tabs,
;       colons, etc.
;
;       The file may contain any number of comment lines - which MUST
;       begin with a semicolon, and MUST be positioned before all data
;       lines.
;
; CALLING SEQUENCE:
; 
;	EROM,V0[,V1,V2,...V9]
;
;       or
;
;       EROM,V=V
;	     
; KEYWORD PARAMETERS:
;
;   V - Set this keyword to a named variable that will be returned as
;       an array of structures holding the data and the variable names
;       specified in the last comment line.  See RESTRICTIONS below
;       for more details.
; 
;   FILE - String specifying the name of a file; if not supplied, the
;          user is queried.
;   
;   SKIP - The number of lines at the beginning of the file that
;          should be skipped.
;          
;   TAB - Specify /TAB for tab-separated data.  (The default is space-
;         separated data.)  It is only necessary to specify this
;         keyword if the file contains any string data columns.
;         
;   SEPARATOR - A string specifying the character separating the data
;               columns.
;
;   COMMENT - Set this keyword to a named variable that will be
;             returned as a string array holding the comment lines
;             included in the file.
;               
;   GROUP - GROUP_LEADER keyword passed to DIALOG_PICKFILE if FILE is
;           not specified.
;
;   CANCEL - Set this keyword to a named variable that will be
;            returned to indicate if the user pressed the CANCEL
;            button when prompted for a file to read, if the FILE
;            keyword is not set.
;
; OUTPUTS:
;
;       If the V keyword is not used, then the user must specify the
;       correct number of Vi (V0, V1, etc.) output parameters.  ;
;       There must be as many Vi's specified in the call to EROM as
;       there are columns of data.  The V's are double-precision
;       arrays, unless either the TAB or SEPARATOR keyword is
;       specified in which case they are all string arrays.
;
; RESTRICTIONS:
;
;       If EROM is called with the V keyword, then the columns of data
;       contained in the file are returned as double-precision fields
;       in the returned V structure variable.  Use of the V keyword
;       requires that the data file contain at least one comment line,
;       and the last comment line MUST include the names of the data
;       variables separated by the "|" character.
;
;       For example, to read a file using the V keyword containing
;       three columns of 10 rows of data, then the last comment line
;       in the file must look like this:
;
;       ;  First Variable Name | Second One | Another Variable Name
;
;       Thus the V structure returned by EROM will have the following
;       tag names:
;
;       HELP,/STR,V
;
;       V[0].VALUE     DOUBLE  Array[10]
;       V[0].NAME      STRING  'First Variable Name'
;       V[1].VALUE     DOUBLE  Array[10]
;       V[1].NAME      STRING  'Second One'
;       V[2].VALUE     DOUBLE  Array[10]
;       V[2].NAME      STRING  'Another Variable Name'
;	
; MODIFICATION HISTORY:
; 
;	David L. Windt, Bell Labs, March 1990
;
;	January, 1997 - DLW
;         Modified to ignore lines beginning with semicolons, and to
;         accept data separated by tabs, etc.; Removed the notitle and
;         comment keyword; included pickfile to prompt for filenames
;         when not specified.
;
;       June, 1997 - DLW
;         Returned numeric variables are now double-precision instead
;         of floating-point.
;	
;	windt@bell-labs.com
;
;       DLW, May 2003
;
;         Added V, COMMENTS, GROUP and CANCEL keywords.
;         Replaced call to PICKFILE with call to DIALOG_PICKFILE
;
;       davidwindt@gmail.com
;-
pro erom,v0,v1,v2,v3,v4,v5,v6,v7,v8,v9,file=file, $
         tab=tab,skip=skip,separator=separator, $
         v=v,comments=comments,group=group, $
         cancel=cancel

on_error,2
on_ioerror,err

; open the file:
if n_elements(group) eq 0 then group=0L
if keyword_set(file) eq '' then file=dialog_pickfile(/read,dialog_parent=group)
if file eq '' then begin
    cancel=1
    return
endif else cancel=0
openr,lun,file,/get_lun
; determine number of lines of data and number of comment lines:
dum=''
n_pts=0L
n_com=0L
while not eof(lun) do begin
    readf,lun,dum
    if strmid(dum,0,1) eq ';' then n_com=n_com+1 else n_pts=n_pts+1
endwhile

if n_elements(skip) eq 1 then begin
    n_com=n_com+skip
    n_pts=n_pts-skip
endif

if n_com gt 0 then  $
  comments=strarr(n_com)        ; create array to hold comments.
point_lun,lun,0                 ; position pointer to beginning of file.
for i=0,n_com-1 do begin
    readf,lun,dum               ; read comments.
    comments[i]=dum             ; save comment lines.
endfor

;; extract variable names:
if n_params() eq 0 then begin
    ;; the variable names must be specified in the last comment line:
    var_name=strtrim(strsplit(strmid(dum,2,strlen(dum)),'|',/extract),2)
endif 

;; number of columns of data:
n_vars=n_params() eq 0 ? n_elements(var_name) : n_params()

;; read data:
if keyword_set(tab) then separator=string(9B)
if keyword_set(separator) then begin
    ;; string data:
    var=strarr(n_vars,n_pts)    ; create string array to store data.
    dum=''                      ; create dummy string to read line.
                                ; read in data:
    for j=0,n_pts-1 do begin    ; loop through each line
        readf,lun,dum           ; read line
        for i=0,n_vars-1 do begin ; loop through each var.
            p1=0                ; initialize position ctr.
            p2=strpos(dum,separator,p1) ; get next tab.
            if p2 eq -1 then p2=strlen(dum) ;
            var[i,j]=strmid(dum,p1,p2) ; extract field.
            dum=strmid(dum,p2+1,strlen(dum)-p2) ; shorten dum.
        endfor
    endfor
endif else begin
    ;; numeric data:
    dum=strarr(n_pts)
    readf,lun,dum
    var=dblarr(n_vars,n_pts)
    reads,dum,var
endelse

if n_params() eq 0 then begin
    ;; if no parameters are passed, then return an array of structures
    ;; containing the data columns and their names in the V keyword:
    v=replicate({value:dblarr(n_pts),name:''},n_vars)
    for i=0,n_vars-1 do begin
        v[i].value=var[i,*]
        v[i].name=var_name[i]
    endfor
endif else begin
    ;; return data columns as vectors, 
    if n_params() ge 1 then v0=reform(var[0,*])
    if n_params() ge 2 then v1=reform(var[1,*])
    if n_params() ge 3 then v2=reform(var[2,*])
    if n_params() ge 4 then v3=reform(var[3,*])
    if n_params() ge 5 then v4=reform(var[4,*])
    if n_params() ge 6 then v5=reform(var[5,*])
    if n_params() ge 7 then v6=reform(var[6,*])
    if n_params() ge 8 then v7=reform(var[7,*])
    if n_params() ge 9 then v8=reform(var[8,*])
    if n_params() gt 9 then v9=reform(var[9,*])
endelse

free_lun,lun
return

err: message,'Error reading file '+file,/info
if n_elements(lun) eq 1 then free_lun,lun
return
end
