;+
; NAME:
; 
;	FLOYD_SAMPLING
;	
; PURPOSE:
; 
;       Randomly choose a unique set of M integers out of a set of N
;       integers ranging in value from 0 to N-1.
;
;       This program uses a sampling algorithm invented by Robert Floyd.
;	
; CALLING SEQUENCE:
; 
;       Result=FLOYD_SAMPLING(SEED,M,N)
;
; INPUTS:
;
;       SEED = A variable or constant used by the call to RANDOMU to
;       initialize the random sequence on input, and in which the
;       state of the random number generator is saved on output. Keep
;       in mind that the number sequences this function returns will
;       not be random if called repeatedly SEED with undefined.
;
;       M = number of integers to select randomly. M must be greater
;       than or equal to 1.
;
;       N = number of integers from which to select (i.e., ranging
;       from 0 to N-1).  N must be greater than or equal to 2, and
;       must be greater than M.
;
; OUTPUTS:
;
;       Result = M-element array of randomly selected integers.
;
; EXAMPLE:
;
;       Result=FLOYD_SAMPLING(5,100)
;
;       Result is a 5-element integer array containing possible values from 0
;       to 99, with no duplicates. It might look like this: Result=[81,3,24,71,60]
;
; MODIFICATION HISTORY:
;
;       David L. Windt, RXO, April 2013
;       davidwindt@gmail.com
;-
function floyd_sampling,seed,m,n

;; called correctly?
if n_params() ne 3 then message,'usage: Result=FLOYD_SAMPLING(SEED,M,N)'

;; make sure m and n are single-valued positive integers:
if (n_elements(m) ne 1) or $
  (n_elements(n) ne 1) or $
  (max(m) ge max(n)) or $
  (min(m) lt 1) or $
  (min(n) lt 2) then message,'invalid input parameters'

;; create array to hold result:
result=intarr(m)-1

;; initialize counting index:
index=0

;; selected random numbers one at a time:
for k=n-m,n-1 do begin

    ;; genererate a random number from 0 to k:
    random_index=fix((1.+k)*randomu(seed))

    ;; if random_index is not already selected, then keep it; if it is
    ;; already selected, then keep k:

    wh=where(result eq random_index,count)
    result[index++]=count eq 0 ? random_index : k
endfor

return,result
end
