;+
; NAME:
;
;      FWHM
;      
; PURPOSE:
; 
;       Interactively measure the full-width-half-max of a region of a
;       curve that has been previously plotted.
;	
; CALLING SEQUENCE:
; 
;	RESULT=FWHM(XAXIS,YAXIS)
;	
; INPUTS:
; 
;	XAXIS - The x axis variable which has been plotted.
;	
;	YAXIS - The y axis variable which has been plotted.
;	
; OPTIONAL INPUT PARAMETERS:
; 
;       RANGE - Vector of subscripts, which refers to the range of X,Y
;               values over which the FWHM is to be determined.  If
;               not supplied, then GET_ROI is used to interactively
;               define the range.  To use FWHM with a non-interactive
;               graphics device, range MUST be supplied.
;	    
; KEYWORD PARAMETERS:
;
;       CWHM - The center point of the peak, defined as the mid-point
;              of the FWHM region.
; 
;       YZERO - The zero point level. If not specified, the zero point
;               level is determined from the endpoints of the region
;               of interest of the curve.
;		
;       YHM - The value at which the full-width is computed.
;             Allowable range is 0. to 1. If not specified, .5 is
;             used.
;		
;       INVERT - Set to get width of 'absorption line' rather than
;                'emission line'.
;		
;       NOHIGHLIGHT - Set to inhibit highlighting the region of
;                     interest.
;		
;       H_COLOR - The color index for highlighting the region of
;                 interest. Default is 7 (Yellow.)
;		
;       H_THICK - The thickness for highlighting the region of
;                 interest.
;		
;	NOLABEL - Set to inhibit labelling the fwhm.
;	
;       L_HEADER - String specifying the label header. Default=''.
;	
;       L_COLOR - Color index for the label.
;	
;       L_FORMAT - Format string for label (eg. '(F4.2)').
;	
;       UNITS - String specifying units along x axis, used in label.
;	
;       CHARSIZE - Size of label text.
;	
;       PSYM - PSYM
;
;       L_CWHM - Set to include CWHM value in label.
;   
; OUTPUTS:
;
;        Result - The full-with-half-max of the region of interest 
;                 of the curve, in x-axis data units.
;		
; OPTIONAL OUTPUT PARAMETERS:
; 
;	ROI - The subscripts of the digitized region of interest.
;	
;       FWHM_ROI - The subscripts of the region between the fwhm
;                  points and the max (min) of the function.
;	    
;       LINE_PTS - A 4-element array containing the coordinates of
;                  the line drawn on the plot: [x0,x1,y0,y1]
;	    
;	LABEL - The label for the plot.
;	
;       L_POS - A two element array containing the x,y coordinates
;               of the label, in data coords.
;
; RESTRICTIONS:
; 
;	The data must be plotted prior to calling FWHM.
;	
; PROCEDURE:
; 
;       The user is asked to digitize the endpoints of the region of
;       interest with the mouse.  The region is highlighted, and the
;       fwhm is labelled.
;	
; MODIFICATION HISTORY:
; 
;	D. L. Windt, Bell Laboratories, November 1989
;
;	March 1998 - Removed MANUAL keyword. 
;
;       January 2004 - Now using local slopes to interpolate fwhm
;                      points, for greater precision.
;
;                    - Added CWHM keyword.
;
;       windt@astro.columbia.edu
;-

function fwhm,xaxis,yaxis,invert=invert,yzero=yzero,yhm=yhm, $
              cwhm=cwhm, $
              roi=roi,fwhm_roi=fwhm_roi,psym=psym,line_pts=line_pts, $
              nohighlight=nohighlight,h_color=h_color,h_thick=h_thick, $
              nolabel=nolabel,manual=manual,units=units, $
              charsize=charsize,l_header=l_header,l_color=l_color,label=label, $
              l_pos=l_pos,l_format=l_format,range=range,l_cwhm=l_cwhm
on_error,2
if n_params() ne 2 then message,'usage: result=fwhm(x,y)'

;; get the region of interest:

if keyword_set(h_color) eq 0 then h_color=7
if keyword_set(h_thick) eq 0 then h_thick=!p.thick
if keyword_set(psym) eq 0 then psym=!p.psym

if keyword_set(range) eq 0 then $
  roi=get_roi(xaxis,yaxis,nohighlight=keyword_set(nohighlight), $
              h_color=h_color,h_thick=h_thick,psym=psym) else roi=range
xroi=float(xaxis[roi])
yroi=float(yaxis[roi])

n_roi=n_elements(roi)		; get number of elements in roi.

;; get zero point and 'half-max' point:
if n_elements(yzero) eq 0 then yzero=.5*(yroi[0]+yroi[n_roi-1])
if n_elements(yhm) eq 1 then yhm=(yhm>0.)<1. else yhm=.5

if keyword_set(invert) then begin ; look at inverted fwhm.
    ymin=min(yroi)              ; get min.
    ymax=yzero                  ; get max.
    ythresh=ymax-yhm*(ymax-ymin) ; threshhold
    fwhm_roi=where(yroi le ythresh) ; region below threshhold
endif else begin                ; look at 'normal' fwhm.
    ymax=max(yroi)              ; get max.
    ymin=yzero                  ; get min.
    ythresh=(yhm*(ymax-ymin)+ymin) ; threshhold
    fwhm_roi=where(yroi ge ythresh) ; region above threshhold
endelse

;; define points on either side of the 1st hm value:
if fwhm_roi[0] eq 0 then begin
    pt0=fwhm_roi[0]
    pt1=fwhm_roi[0]+1
endif else begin
    pt0=fwhm_roi[0]-1
    pt1=fwhm_roi[0]
endelse

;; if pts are invalid, then return:
if (pt0 eq -1) or (pt1 eq -1) or $
  (pt1 gt (n_elements(yroi)-1)) then begin
    cwhm=-1
    return,-1
endif

;; compute slope of curve at the hm value:
slope1=(yroi[pt1]-yroi[pt0])/(xroi[pt1]-xroi[pt0])

;; interpolated value of x for which y=yhm exactly:
x1=xroi[pt1]-(yroi[pt1]-ythresh)/slope1

;; define points on either side of the last hm value:
n=n_elements(fwhm_roi)-1
n_pts=n_elements(yroi)
if fwhm_roi[n] eq n_pts-1 then begin
    pt0=fwhm_roi[n]-1
    pt1=fwhm_roi[n]
endif else begin
    pt0=fwhm_roi[n]
    pt1=fwhm_roi[n]+1
endelse

;; compute slope of curve at the hm value:
slope2=(yroi[pt1]-yroi[pt0])/(xroi[pt1]-xroi[pt0])

;; interpolated value of x for which y=yhm exactly:
x2=xroi[pt0]+(ythresh-yroi[pt0])/slope2

;; now ensure that x2 > x1:

x1_temp=x1
x2_temp=x2
x1=x1_temp < x2_temp
x2=x1_temp > x2_temp

;; get fwhm in data coordinates:
fwhm=x2-x1

;; get center point:
cwhm=(x1+x2)/2.

;; label the line...
if keyword_set(l_color) eq 0 then l_color=1	
if keyword_set(l_header) eq 0 then l_header=''
if keyword_set(units) eq 0 then units=''
if keyword_set(charsize) eq 0 then charsize=1 > !p.charsize

;; make label:
if keyword_set(l_format) eq 0 then begin
    label=l_header+strtrim(string(fwhm),2)+' '+units 
    if keyword_set(l_cwhm) then label=label+' (CWHM='+strtrim(string(cwhm),2)+' '+units+')'
endif else begin
    label=l_header+strtrim(string(fwhm,format=l_format),2)+' '+units
    if keyword_set(l_cwhm) then label=label+' (CWHM='+strtrim(string(cwhm,format=l_format),2)+' '+units+')'
endelse

;; make label position and place the label:
l_pos=[.1*(x2-x1)+x2,ythresh]

if keyword_set(nolabel) eq 0 then begin 
    ;; plot a line across the roi:
    line_pts=[x1,x2,ythresh,ythresh]
    plots,[x1,x2],[ythresh,ythresh],color=l_color,noclip=0
    ;; display the label
    xyouts,l_pos[0],l_pos[1],label,color=l_color,size=charsize
endif else label=(keyword_set(invert) ? 'FWHMin' : 'FWHM')+'='+label

; redefine fwhm_roi relative to roi:
fwhm_roi=roi[0]+fwhm_roi

return,fwhm	
end

