;+
; NAME:
;
;      MK_NEW_PTRS
;      
; PURPOSE:
;
;       Make a copy of a pointer variable, or a structure variable
;       containing pointer variables as tags, such that the
;       de-referenced pointer values are copied, but new pointer heap
;       variables are created in the copy of the original variable.
;
; CALLING SEQUENCE:
;
;       Results=MK_NEW_PTRS(SOURCE)
;
; INPUTS:
;
;       SOURCE = The source structure variable.
;
; EXAMPLE:
;
;       IDL> a={n:ptr_new(/allocate_heap)}
;       IDL> b=a
;       IDL> help,a.n,b.n
;       <Expression>    POINTER   = <PtrHeapVar1>
;       <Expression>    POINTER   = <PtrHeapVar1>
;       IDL> b=mk_new_ptrs(a)
;       IDL> help,a.n,b.n
;       <Expression>    POINTER   = <PtrHeapVar1>
;       <Expression>    POINTER   = <PtrHeapVar2>
;
; PROCEDURE:
;
;       SAVE and RESTORE to and from a temporary file are used to
;       generate the new pointers.  Ideally, there would be keyword to
;       the STRUCT_ASSIGN procedure that would accomplish the same
;       thing without having to use this workaround...but so far (IDL
;       6.0) such a keyword does not exist.
;
; MODIFICATION HISTORY:
; 
;	David L. Windt, Columbia University, Oct-2003
;
;       windt@astro.columbia.edu
;-
function mk_new_ptrs,source

on_error,2

if n_params() eq 0 then begin
    message,'Usage: Result=MK_NEW_PTRS(Source)'
    return,0
endif

;; copy variable
dest=source

;;initialize counter:
i=0
while keyword_set(file_ok) eq 0 do begin

    if !version.os_family eq 'unix' then dir='/tmp' else $
      dir='/temp'

    ;; check temp directory:
    dfi=file_info(dir)

    ;; if /tmp is not writeable, error:
    if dfi.write eq 0 then message,dir+' directory is write-protected!'

    ;; make a temporary file name:
    file=dir+'/idl'+strtrim(i,2)+'.sav'
    
    ;; get file info:
    fi=file_info(file)

    ;; make sure we can write to the temporary file:
    file_ok=((fi.exists eq 0) or $
             ((fi.exists eq 1) and (fi.write)))

    ;; increment counter:
    i++
endwhile

;; use save/restore to make new pointers:
save,dest,file=file
restore,file=file
file_delete,file

return,dest
end


