;+
; NAME:
;
;      PTRS_NEW
;      
; PURPOSE:
;
;      This function will make a copy of the passed variable, which
;      can be of any type. However if it's a pointer, an array of
;      pointers, a structure, or any array of structures, then the
;      entire hierarchy will be searched until all pointers are
;      replaced with new pointers, so that the copy will not point to
;      anything that the original passed variable points to.
;
; CALLING SEQUENCE:
;
;       DEST=PTRS_NEW(SOURCE)
;
; INPUTS:
;
;       SOURCE: Variable of any type.
;
; OUTPUTS:
;
;       DEST: the destination variable, which is an exact copy of the
;       source variable SOURCE, except that all pointers contained in
;       the returned variable will be new.
;
; MODIFICATION HISTORY:
; 
;	David L. Windt, Reflective X-ray Optics, davidwindt@gmail.com
;	14-May-2013
;
;-

function ptrs_new,var,allocate_heap=allocate_heap

on_error,2

if n_elements(var) eq 0 then return,0

;; make a copy of passed variable:
new_var=var

;; shorthand:
allocate_heap=keyword_set(allocate_heap)

;; var can be an array, so check each element:
for i=0,n_elements(new_var)-1 do begin

    case 1 of 
        ;; var is a pointer:
        size(new_var[i],/type) eq 10: begin

            ;; make a new pointer:
            new_var[i]=ptr_valid(new_var[i]) ? ptr_new(*new_var[i]) : ptr_new(allocate_heap=allocate_heap)

            ;; now see if this variable points to another pointer, an
            ;; array of pointers, a structure, or an array of structures,
            ;; any of which might contain more pointers:

            ;; valid, non-null pointer?
            if ptr_valid(new_var[i]) then $
              if n_elements(*new_var[i]) gt 0 then begin

                ;; dereference:
                d_new_var_i=*new_var[i]

                ;; if it's a pointer or a structure, then call ourself:
                if (size(d_new_var_i,/type) eq 10) or (size(d_new_var_i,/type) eq 8) then begin

                    d_new_var_i=ptrs_new(d_new_var_i,allocate_heap=allocate_heap)

                    ;; re-reference:
                    *new_var[i]=d_new_var_i
                endif
            endif
        end

        ;; var is a structure:
        size(new_var[i],/type) eq 8: begin

            ;; check each tag:
            for j=0,n_tags(new_var[i])-1 do begin

                ;; tag variable:
                tag_j=new_var[i].(j)

                ;; tag is a pointer, a pointer array, a structure, or
                ;; a structure array:
                if (size(tag_j,/type) eq 10) or (size(tag_j,/type) eq 8) then begin

                    ;; call ourself:
                    tag_j=ptrs_new(tag_j,allocate_heap=allocate_heap)
                    
                    ;; save new tag:
                    new_var[i].(j)=tag_j
                endif

            endfor
        end

        ;; var is neither, so do nothing more:
        else: ;;
    endcase
endfor

return,new_var
end

