;+-----------------------------------------------------------------------------
;  NAME:
;	TWOSCOMPLEMENT
;
;  PURPOSE:
; 	Taking the Two's Complement of an integer
;
; CATEGORY:
;	Math, Hardware, CAMAC
;
; CALLING SEQUENCE:
;       twoscomp = TwosComplement( int )
;
; INPUT:
;       int - raw encoder value (8, 16 or 32 bit integer)
;
; OUTPUT:
;       twoscomp - Two's complement of input.
;
; KEYWORDS:
;    Optional Input:
;	NBITS - # of bits; throw away this bit if there is a carry after 
;		adding 1 to the complement. Default is determined by data type.
;	IfNeg - Only return the Two's Complement if value negative
; ALGORITHM:
;
;	Taking the Two's Complement of a k-Digit Bitstring:
;
;   1.Complement the bitstring; i.e., change all 0s to 1s and all 1s to 0s; retain all leading 0s in your result. 
;   2.Add 1 to this binary number  (if there is a carry of 1 into the (k+1)st position, throw it away so that the
;     result is still k-digits). 
;   3.The result from (2) is the two's complement of the bitstring 
;
; COMMENT:
;	Works in many cases, but sign bit may get extended in some applications
;  MODIFICATION HISTORY:
; 	5-Jun-00 WMD Added Nbits & IfNeg Keywords
;------------------------------------------------------------------------------
FUNCTION twoscomplement, input, NBITS=nBits, IfNeg=IfNeg, QUIET=Quiet

b = SIZE(input)
IF b(0) GT 0 THEN BEGIN
   IF b(1) GT 1 THEN BEGIN
	  bell
	  PRINT, ' '
	  PRINT, " *** twoscomplement routine only works with scalars ***"
	  PRINT, ' '
   ENDIF
ENDIF

bitArray = mk_bitarray(input, NBITS=nBits )

a =  (bitarray+1) mod 2                        ; flip 1's and 0's

twos = 2L ^ INDGEN(31)

ones = TOTAL( a * twos )	; form new number 
longA = ones + 1

; now throw away any carried 1, and return same type of variable

infoArray = SIZE( input )

icheck = 1
typeCodeLoc = N_ELEMENTS( infoArray ) - 2
IF infoArray(typeCodeLoc) EQ 1 THEN BEGIN
   mask='FF'X
   b = BYTE( longA )
ENDIF ELSE IF infoArray(typeCodeLoc) EQ 2 THEN BEGIN
   mask='FFFF'X
   b = FIX( longA )
ENDIF ELSE IF infoArray(typeCodeLoc) EQ 3 THEN BEGIN
   mask='FFFFFFFF'X
   b = LONG( longA )
ENDIF ELSE BEGIN
   IF NOT KEYWORD_SET( Quiet ) THEN print, ' twoscomplement only works on integers '
   mask='FFFFFFFF'X
   b = LONG( longA )
ENDELSE

IF N_ELEMENTS( nBits ) GT 0 THEN BEGIN
   mask = 2L ^ nBits - 1
ENDIF

b = b AND mask 


If KEYWORD_SET( IfNeg ) THEN BEGIN	; replace orignal positive values
   IF infoArray(0) EQ 0 THEN BEGIN
      IF input GE 0 THEN return, input
   ENDIF ELSE BEGIN
      inds = WHERE (input GE 0, npos)
      IF npos GT 0 THEN b(inds) = input(inds)
   ENDELSE
ENDIF

return, b

end
